/*
 * ********************************************************************************************************************
 *
 *                     :*$@@%$*:                         ;:                ;;    ;;
 *                   :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                  :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                  =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                  =@*                                  %!              @ $= % %@=   =%@!      %=
 *             *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *           %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *         ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *         $@*   ;@@@%=!:                *@*
 *         =@$    ;;;!=%@@@@=!           =@!
 *          %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *           ;%@@$=$@@%*       *@@@$=%@@%;
 *              ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.persagy.server.service.objects

import com.persagy.server.utils.ShaftNameUtil
import com.persagy.database.SFilter
import com.persagy.database.SQueryBuilder
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.objects.Property
import com.persagy.server.service.EquipFamilyService
import com.persagy.server.service.base.Service
import com.persagy.server.utils.IdUtils
import com.persagy.service.SPageContext
import org.slf4j.LoggerFactory

/**
 * 资产信息服务
 *
 * @author 张维新
 */
object PropertyService : Service<Property>(SMybatisDao(Property::class.java)) {
    /** 日志 */
    private val logger = LoggerFactory.getLogger(PropertyService::class.java)

    /**
     * 创建之前的操作
     *
     * @param   property    创建的对象
     */
    override fun onCreateBefore(property: Property): Boolean  {
        if (!property.id.isNullOrEmpty()) {
            val isStartsWith = property.id.toString().startsWith("Pe")
            if (!isStartsWith) {
                /** 资产id */
                property.id = IdUtils.peIdCreate(property.projectId!!)
            }
        } else {
            /** 资产id */
            property.id = IdUtils.peIdCreate(property.projectId!!)
        }
//        /** 设备名字 */
//        if (property.name.isNullOrEmpty()) {
//            property.name = ShaftNameUtil.shaftName("资产-")
//        }
        if (!property.family.isNullOrEmpty()){
            val family = EquipFamilyService.select(SFilter.eq("family", property.family!!)).entity()
            if (family!=null){
                /** 资产名字 */
                if (property.name.isNullOrEmpty()) {
                    property.name = ShaftNameUtil.shaftName(family.familyName+"-")
                }
            }else{
                /** 资产名字 */
                if (property.name.isNullOrEmpty()) {
                    property.name = ShaftNameUtil.shaftName("资产-")
                }
            }
            return true
        }else{
            return false
        }
    } // Function onCreateBefore()

    /**
     * 查询之前的操作
     *
     * @param   queryBuilder    查询条件
     */
    override fun onQueryBefore(queryBuilder: SQueryBuilder<Property>) {
        queryBuilder.tableName = "v_property_equip"
        return
    } // Function onQueryBefore()

    override fun onUpdateBefore(entity: Property): Boolean {
        val projectId = SPageContext.getHeader("projectId")
        entity.projectId = projectId
        return super.onUpdateBefore(entity)
    } // Function onUpdateBefore()

} // Object PropertyService
